use std::{error::Error, io::prelude::*, net::TcpStream};

use super::requests::Request;

pub enum Body {
    String(String),
    Static(&'static str),
    Bytes(Vec<u8>, &'static str),
}

pub struct Response {
    request: Request,
    pub body: Body,
    pub status: u16,
}

impl Response {
    pub fn new(request: Request, status: u16, body: Body) -> Self {
        Response {
            request,
            body,
            status,
        }
    }

    pub fn send(self) -> UnitOrBoxedError {
        send_response(self.request.mut_stream(), self.status, self.body)
    }
}

pub type UnitOrBoxedError = Result<(), Box<dyn Error>>;

fn send_response(mut stream: TcpStream, status: u16, body: Body) -> UnitOrBoxedError {
    let status_line = match status {
        200 => "200 OK",
        400 => "400 BAD REQUEST",
        404 => "404 NOT FOUND",
        405 => "405 METHOD NOT ALLOWED",
        _ => "500 INTERNAL SERVER ERROR",
    };

    let mut response = format!("HTTP/1.1 {}\r\nServer: luciders/0.1.0\r\n", status_line);

    // Headers
    response.push_str(&format!(
        "Content-Length: {}\r\n",
        match &body {
            Body::String(s) => s.len(),
            Body::Static(s) => s.len(),
            Body::Bytes(b, _) => b.len(),
        }
    ));
    if let Body::Bytes(_, content_type) = &body {
        response.push_str(&format!("Content-Type: {}\r\n", content_type));
    }
    response.push_str("\r\n");
    
    // Body
    match body {
        Body::String(s) => response.push_str(&s),
        Body::Static(s) => response.push_str(s),
        Body::Bytes(b, _) => {
            for byte in b {
                response.push(byte as char);
            }
        }
    }

    match stream.write(response.as_bytes()) {
        Ok(_) => {}
        Err(e) => {
            return Err(Box::new(e));
        }
    }

    match stream.flush() {
        Ok(_) => {}
        Err(e) => {
            return Err(Box::new(e));
        }
    }

    Ok(())
}
